==> apache-asterixdb-0.9.6-source-release.zip: apache-asterixdb-0.9.6/asterix-metadata/src/main/java/org/apache/asterix/metadata/functions/ExternalTypeComputer.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.asterix.metadata.functions;

import java.util.List;

import org.apache.asterix.om.exceptions.TypeMismatchException;
import org.apache.asterix.om.typecomputer.base.AbstractResultTypeComputer;
import org.apache.asterix.om.types.ATypeTag;
import org.apache.asterix.om.types.AUnionType;
import org.apache.asterix.om.types.IAType;
import org.apache.asterix.om.types.hierachy.ATypeHierarchy;
import org.apache.hyracks.algebricks.common.exceptions.AlgebricksException;
import org.apache.hyracks.algebricks.core.algebra.base.ILogicalExpression;
import org.apache.hyracks.algebricks.core.algebra.functions.FunctionIdentifier;
import org.apache.hyracks.api.exceptions.SourceLocation;

public class ExternalTypeComputer extends AbstractResultTypeComputer {

    private IAType resultType;
    private List<IAType> paramPrimeTypes;

    @Override
    protected void checkArgType(FunctionIdentifier funcId, int argIndex, IAType type, SourceLocation sourceLoc)
            throws AlgebricksException {
        IAType reqParamType = paramPrimeTypes.get(argIndex);
        if (!type.equals(paramPrimeTypes.get(argIndex))
                && !ATypeHierarchy.isCompatible(type.getTypeTag(), reqParamType.getTypeTag())) {
            throw new TypeMismatchException(sourceLoc, funcId, argIndex, type.getTypeTag(),
                    paramPrimeTypes.get(argIndex).getTypeTag());
        }
    }

    public ExternalTypeComputer(IAType resultPrimeType, List<IAType> paramPrimeTypes) {
        this.resultType = resultPrimeType.getTypeTag() == ATypeTag.ANY ? resultPrimeType
                : AUnionType.createUnknownableType(resultPrimeType);
        this.paramPrimeTypes = paramPrimeTypes;
    }

    @Override
    protected IAType getResultType(ILogicalExpression expr, IAType... strippedInputTypes) {
        return resultType;
    }
}

==> apache-asterixdb-0.9.7-source-release.zip: apache-asterixdb-0.9.7/asterix-metadata/src/main/java/org/apache/asterix/metadata/functions/ExternalTypeComputer.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.asterix.metadata.functions;

import java.util.List;

import org.apache.asterix.om.exceptions.TypeMismatchException;
import org.apache.asterix.om.typecomputer.base.AbstractResultTypeComputer;
import org.apache.asterix.om.types.ATypeTag;
import org.apache.asterix.om.types.AUnionType;
import org.apache.asterix.om.types.IAType;
import org.apache.asterix.om.types.hierachy.ATypeHierarchy;
import org.apache.hyracks.algebricks.common.exceptions.AlgebricksException;
import org.apache.hyracks.algebricks.core.algebra.base.ILogicalExpression;
import org.apache.hyracks.algebricks.core.algebra.functions.FunctionIdentifier;
import org.apache.hyracks.api.exceptions.SourceLocation;

public class ExternalTypeComputer extends AbstractResultTypeComputer {

    private final IAType resultType;
    private final List<IAType> paramPrimeTypes;
    private final boolean nullCall;

    @Override
    protected void checkArgType(FunctionIdentifier funcId, int argIndex, IAType type, SourceLocation sourceLoc)
            throws AlgebricksException {
        IAType reqParamType = paramPrimeTypes.get(argIndex);
        if (!type.equals(paramPrimeTypes.get(argIndex))
                && !ATypeHierarchy.isCompatible(type.getTypeTag(), reqParamType.getTypeTag())) {
            throw new TypeMismatchException(sourceLoc, funcId, argIndex, type.getTypeTag(),
                    paramPrimeTypes.get(argIndex).getTypeTag());
        }
    }

    public ExternalTypeComputer(IAType resultPrimeType, List<IAType> paramPrimeTypes, boolean nullCall) {
        this.resultType = resultPrimeType.getTypeTag() == ATypeTag.ANY ? resultPrimeType
                : AUnionType.createUnknownableType(resultPrimeType);
        this.paramPrimeTypes = paramPrimeTypes;
        this.nullCall = nullCall;
    }

    @Override
    protected IAType getResultType(ILogicalExpression expr, IAType... strippedInputTypes) {
        return resultType;
    }

    @Override
    protected boolean propagateNullAndMissing() {
        return !nullCall;
    }
}
